/*# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image
#                        pour la Sant�)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
# Previous Authors : Laurent Guigues, Jean-Pierre Roux
# CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
#
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------ */

#include "manualRotationToolControler.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

// _state = 0  // ..nothing..
// _state = 5  // move point
// _state = 6  // move all
// _state = 7  // Empty mouse drag

manualRotationToolControler::manualRotationToolControler()
{

}

// ----------------------------------------------------------------------------
manualRotationToolControler::~manualRotationToolControler()
{

}

// ----------------------------------------------------------------------------
manualRotationToolControler * manualRotationToolControler :: Clone()  // virtual
{
	manualRotationToolControler * clone = new manualRotationToolControler();
	CopyAttributesTo(clone);
	return clone;
}

// ---------------------------------------------------------------------------
void manualRotationToolControler::CopyAttributesTo( manualRotationToolControler * cloneObject)
{
	// Fathers object
	manualContourBaseControler::CopyAttributesTo(cloneObject);
}

// ----------------------------------------------------------------------------
void manualRotationToolControler::Configure() //virtual
{

}

// ----------------------------------------------------------------------------
void manualRotationToolControler::MouseClickLeft(int x, int y)
{
	int z = GetZ();
	
	if( IsEditable() )
	{	// move control point
  	  if ( (GetState()==0) && (GetManualViewBaseContour()->GetIdPoint(x,y,z)!=-1 ) ){
  		  bakIdPoint = GetManualViewBaseContour()->GetIdPoint(x,y,z);
		  SetState(5);
	  }
	}// IsEditable
	
	// move contour
	if ((GetState()==0) && (GetManualViewBaseContour()->GetPosibleSelected()==true))	{
		GetManualViewBaseContour()->InitMove(x,y,z);
		SetState(6);
	}
	
	// firstime create 3 control points and move one control point
	int size=GetManualViewBaseContour()->GetNumberOfPoints();
	if (GetState()==0) {
		if (size==0){
			AddPoint(x,y,z);
			AddPoint(x,y,z);
			bakIdPoint = GetManualViewBaseContour()->GetIdPoint(x,y,z);
			AddPoint(x,y,z);
			SetState(1);
		}
	}	
	
	GetManualViewBaseContour()->Refresh();
}

// ----------------------------------------------------------------------------
void manualRotationToolControler::MouseMove(int x, int y) // virtual
{
	int z=GetZ();

	GetManualViewBaseContour()->SelectPosibleContour(x,y,z);
	GetManualViewBaseContour()->SelectPosiblePoint(x,y,z);

	if (GetState()==1){	SetPoint( bakIdPoint , x , y ,z); }
	if (GetState()==5){	SetPoint( bakIdPoint , x , y ,z); }
	
	if (GetState()==6){
		GetManualViewBaseContour()->MoveContour(x,y,z);
	}
	GetManualViewBaseContour()->Refresh();
}

// ----------------------------------------------------------------------------
void manualRotationToolControler::DeleteActualMousePoint(int x, int y)  // virtual
{
}

// ----------------------------------------------------------------------------
void manualRotationToolControler::InitRoi(int ww, int hh, double porcentage)
{
	int zz;
	manualPoint *mp;

	if (GetManualContourModel()->GetSizeLstPoints() ==0)
	{
		zz = GetZ();
		AddPoint(0,0,zz);
		AddPoint(0,0,zz);
		AddPoint(0,0,zz);
	}

	double pp1=porcentage;
	double pp2=1-porcentage;

	mp = GetManualContourModel()->GetManualPoint(2);
	zz=(int)mp->GetZ();
	mp->SetPoint(ww*pp1,hh*pp1,zz);

	mp = GetManualContourModel()->GetManualPoint(1);
	zz=(int)mp->GetZ();
	cout <<  ww*pp2 <<hh*pp1 << zz<<endl;
	mp->SetPoint(ww*pp2,hh*pp1,zz);

	mp = GetManualContourModel()->GetManualPoint(0);
	zz=(int)mp->GetZ();
	cout <<  ww*pp2 <<hh*pp2 << zz<<endl;
	mp->SetPoint(ww*pp2,hh*pp2,zz);

	GetManualViewBaseContour() ->UpdateViewPoint(0);
	GetManualViewBaseContour() ->UpdateViewPoint(1);
	GetManualViewBaseContour() ->UpdateViewPoint(2);

	SetState(0);
	GetManualViewBaseContour()->Refresh();
}
